@@ -53,7 +53,6 @@ struct connman_device {
 							 */
 	bool powered;
 	bool scanning;
-	bool disconnected;
 	char *name;
 	char *node;
 	char *address;
@@ -568,7 +567,7 @@ int connman_device_set_powered(struct connman_device *device,
 {
 	enum connman_service_type type;
 
-	DBG("driver %p powered %d", device, powered);
+	DBG("device %p powered %d", device, powered);
 
 	if (device->powered == powered)
 		return -EALREADY;
@@ -588,7 +587,6 @@ int connman_device_set_powered(struct connman_device *device,
 
 	__connman_technology_enabled(type);
 
-	connman_device_set_disconnected(device, false);
 	device->scanning = false;
 
 	if (device->driver && device->driver->scan)
@@ -623,8 +621,6 @@ int __connman_device_disconnect(struct connman_device *device)
 
 	DBG("device %p", device);
 
-	connman_device_set_disconnected(device, true);
-
 	g_hash_table_iter_init(&iter, device->networks);
 
 	while (g_hash_table_iter_next(&iter, &key, &value)) {
@@ -750,37 +746,6 @@ int connman_device_set_scanning(struct connman_device *device,
 	return 0;
 }
 
-/**
- * connman_device_set_disconnected:
- * @device: device structure
- * @disconnected: disconnected state
- *
- * Change disconnected state of device (only for device with networks)
- */
-int connman_device_set_disconnected(struct connman_device *device,
-						bool disconnected)
-{
-	DBG("device %p disconnected %d", device, disconnected);
-
-	if (device->disconnected == disconnected)
-		return -EALREADY;
-
-	device->disconnected = disconnected;
-
-	return 0;
-}
-
-/**
- * connman_device_get_disconnected:
- * @device: device structure
- *
- * Get device disconnected state
- */
-bool connman_device_get_disconnected(struct connman_device *device)
-{
-	return device->disconnected;
-}
-
 /**
  * connman_device_set_string:
  * @device: device structure
